<?php
/**
 * Theme functions and definitions.
 *
 * @package Stag_Customizer
 */

/**
 * Set the content width based on the theme's design and stylesheet.
 */
if ( ! isset( $content_width ) ) {
	$content_width = 770;
}

if ( ! function_exists( 'stag_theme_setup' ) ) :
	/**
	 * Sets up theme defaults and registers support for various WordPress features.
	 *
	 * Note that this function is hooked into the after_setup_theme hook, which
	 * runs before the init hook. The init hook is too late for some features, such
	 * as indicating support for post thumbnails.
	 */
	function stag_theme_setup() {
		// Attempt to load text domain from child theme first.
		if ( ! load_theme_textdomain( 'skilt', get_stylesheet_directory() . '/languages' ) ) {
			load_theme_textdomain( 'skilt', get_template_directory() . '/languages' );
		}

		// Feed links.
		add_theme_support( 'automatic-feed-links' );

		// Featured images.
		add_theme_support( 'post-thumbnails' );

		// Post formats.
		$formats = array(
			'aside',
			'link',
			'quote',
			'image',
			'gallery',
			'audio',
			'video',
		);

		add_theme_support( 'post-formats', $formats );

		// HTML5.
		add_theme_support( 'html5', array( 'comment-list', 'search-form', 'comment-form', 'gallery', 'caption' ) );

		// Menu locations.
		register_nav_menus(
			array(
				'primary' => __( 'Primary Menu', 'skilt' ),
				'footer'  => __( 'Footer Menu', 'skilt' ),
			)
		);

		$background_args = array(
			'default-color'          => '373f4d',
			'default-image'          => get_template_directory_uri() . '/assets/img/default-background.jpg',
			'default-repeat'         => 'no-repeat',
			'default-position-x'     => 'center',
			'wp-head-callback'       => '__return_false',
			'admin-head-callback'    => '__return_false',
			'admin-preview-callback' => '__return_false',
		);
		add_theme_support( 'custom-background', $background_args );

		/**
		* Add theme support for selective refresh for widgets.
		*
		* @since 2.0.1
		*/
		add_theme_support( 'customize-selective-refresh-widgets' );

		/**
		 * Let WordPress manage the document title.
		 * By adding theme support, we declare that this theme does not use a
		 * hard-coded <title> tag in the document head, and expect WordPress to
		 * provide it for us.
		 *
		 * @since 2.0.1
		 */
		add_theme_support( 'title-tag' );

		/**
		 * Gutenberg support.
		 */
		add_theme_support( 'wp-block-styles' );
		add_theme_support( 'responsive-embeds' );
		add_theme_support( 'editor-styles' );

		add_editor_style( '/assets/css/editor-style.css' );
	}
endif;

add_action( 'after_setup_theme', 'stag_theme_setup' );

if ( ! function_exists( 'stag_image_sizes' ) ) :
	/**
	 * Register theme image sizes. Filterable.
	 *
	 * @since 1.0.
	 */
	function stag_image_sizes() {
		$defaults    = array(
			'content-grid' => array( 570, 9999, false ),
			'content-full' => array( 1170, 9999, false ),
		);
		$image_sizes = apply_filters( 'stag_image_sizes', $defaults );
		foreach ( $image_sizes as $name => $prop ) {
			$crop = ( isset( $prop[2] ) ) ? $prop[2] : false;
			add_image_size( $name, $prop[0], $prop[1], $crop );
		}
	}
endif;

add_action( 'after_setup_theme', 'stag_image_sizes' );

if ( ! function_exists( 'stag_widgets_init' ) ) :
	/**
	 * Register widget areas and widgets.
	 *
	 * @since 1.0
	 */
	function stag_widgets_init() {
		register_sidebar(
			array(
				'name'          => __( 'Side Drawer Widget Area', 'skilt' ),
				'id'            => 'sidebar-drawer',
				'before_widget' => '<aside id="%1$s" class="site-nav__section %2$s">',
				'after_widget'  => '</aside>',
				'before_title'  => '<h4 class="widgettitle">',
				'after_title'   => '</h4>',
				'description'   => __( 'Sidebar drawer navigation widgets.', 'skilt' ),
			)
		);
	}
endif;
add_action( 'widgets_init', 'stag_widgets_init' );

/**
 * Enqueue Block Editor Assets.
 */
function skilt_block_editor_styles() {

	// Google fonts.
	$google_request = stag_get_google_font_uri();

	if ( '' !== $google_request ) {
		// Enqueue the fonts.
		wp_enqueue_style(
			'stag-google-fonts',
			$google_request,
			array(),
			STAG_THEME_VERSION
		);
	}

	wp_enqueue_style(
		'font-awesome',
		get_template_directory_uri() . '/assets/css/fontawesome-all.min.css',
		array(),
		'5.0.2'
	);

	$font_header       = stag_theme_mod( 'font', 'header' );
	$font_header_stack = stag_get_font_stack( $font_header );
	$font_body         = stag_theme_mod( 'font', 'body' );
	$font_body_stack   = stag_get_font_stack( $font_body );

	$font_body_size   = absint( stag_theme_mod( 'font', 'body-size' ) );
	$font_header_size = absint( stag_theme_mod( 'font', 'header-size' ) );

	$percent = apply_filters(
		'stag_font_relative_size',
		array(
			// Relative to header font size.
			'h1' => 100,
			'h2' => 68,
			'h3' => 60,
			'h4' => 54,
			'h5' => 48,
			'h6' => 38,
		)
	);

	$h2_font_size = absint( stag_get_relative_font_size( $font_header_size, $percent['h2'] ) );
	$h3_font_size = absint( stag_get_relative_font_size( $font_header_size, $percent['h3'] ) );
	$h4_font_size = absint( stag_get_relative_font_size( $font_header_size, $percent['h4'] ) );
	$h5_font_size = absint( stag_get_relative_font_size( $font_header_size, $percent['h5'] ) );
	$h6_font_size = absint( stag_get_relative_font_size( $font_header_size, $percent['h6'] ) );

	$accent_color      = stag_maybe_hash_hex_color( stag_theme_mod( 'colors', 'accent' ) );
	$header_text_color = stag_maybe_hash_hex_color( stag_theme_mod( 'colors', 'header-area-text-color' ) );

	wp_add_inline_style(
		'stag-google-fonts',
		"
		.editor-styles-wrapper {
			--accent-color: {$accent_color};
			--header-txt-color: {$header_text_color};
			--font-body: {$font_body_stack};
			--font-header: {$font_header_stack};
			--body-font-size: {$font_body_size}px;
			--h1-size: {$font_header_size}px;
			--h2-size: {$h2_font_size}px;
			--h3-size: {$h3_font_size}px;
			--h4-size: {$h4_font_size}px;
			--h5-size: {$h5_font_size}px;
			--h6-size: {$h6_font_size}px;
		}"
	);
}
add_action( 'enqueue_block_editor_assets', 'skilt_block_editor_styles' );

/**
 * Enqueues scripts and styles for front end.
 */
function stag_scripts_styles() {

	$style_dependencies = array();

	// Google fonts.
	$google_request = stag_get_google_font_uri();

	if ( '' !== $google_request ) {
		// Enqueue the fonts.
		wp_enqueue_style(
			'stag-google-fonts',
			$google_request,
			$style_dependencies,
			STAG_THEME_VERSION
		);
		$style_dependencies[] = 'stag-google-fonts';
	}

	// FontAwesome Icons, enqueue only when its not enqueued already.
	if ( ! class_exists( 'StagTools' ) ) {
		wp_enqueue_style(
			'font-awesome',
			get_template_directory_uri() . '/assets/css/fontawesome-all.min.css',
			$style_dependencies,
			'5.0.2'
		);
		$style_dependencies[] = 'font-awesome';
	}

	// Main stylesheet.
	wp_enqueue_style( 'stag-style', get_stylesheet_uri(), $style_dependencies, STAG_THEME_VERSION );

	$script_dependencies = array( 'jquery' );

	// Enqueue Masonry only if needed.
	if ( is_home() || is_archive() || is_search() ) {
		global $wp_version;
		if ( true === version_compare( $wp_version, '3.8.9', '>' ) ) {
			$script_dependencies[] = 'masonry';
		} else {
			$script_dependencies[] = 'jquery-masonry';
		}
	}

	// Enqueue ResponsiveSlides only if needed.
	if ( is_home() || is_archive() || ( is_single() ) ) {
		$script_dependencies[] = 'responsiveslides';
	}

	if ( is_404() ) {
		$script_dependencies[] = 'fittext';
	}

	wp_register_script( 'stag-global', get_template_directory_uri() . '/assets/js/global' . STAG_SCRIPT_SUFFIX . '.js', $script_dependencies, STAG_THEME_VERSION, true );
	wp_register_script( 'stag-plugins', get_template_directory_uri() . '/assets/js/plugins.js', array( 'jquery', 'stag-global' ), STAG_THEME_VERSION, true );
	wp_register_script( 'fittext', get_template_directory_uri() . '/assets/js/lib/fittext/jquery.fittext' . STAG_SCRIPT_SUFFIX . '.js', array( 'jquery' ), '1.1.1', true );
	wp_register_script( 'responsiveslides', get_template_directory_uri() . '/assets/js/lib/responsiveslides/responsiveslides' . STAG_SCRIPT_SUFFIX . '.js', array( 'jquery' ), '1.55', true );

	// Enqueue Scripts.
	wp_enqueue_script( 'stag-plugins' );
	wp_enqueue_script( 'imagesloaded' );

	// Comment reply.
	if ( is_singular() && comments_open() && get_option( 'thread_comments' ) ) {
		wp_enqueue_script( 'comment-reply' ); // loads the javascript required for threaded comments.
	}

	// Send to the script.
	wp_localize_script(
		'stag-global',
		'skilt',
		array(
			'ajaxurl'         => admin_url( 'admin-ajax.php' ),
			'alreadyLikeText' => __( 'You already like this post', 'skilt' ),
		)
	);
}
add_action( 'wp_enqueue_scripts', 'stag_scripts_styles' );

if ( ! isset( $stag_post ) ) {
	$stag_post = array();
}

if ( ! function_exists( 'stag_setup_data' ) ) :
	/**
	 * Setup the content grabber class for post formats.
	 *
	 * This function hooks into the_post and uses its data to create a new instance
	 * of a content grabber class, after first clearing out a previous instance.
	 *
	 * @since  1.0.
	 *
	 * @param  object $post The current post object.
	 * @return void
	 */
	function stag_setup_data( $post ) {
		global $stag_post;

		$stag_post   = array();
		$post_format = get_post_format();
		$id          = absint( $post->ID );

		if ( stag_is_article( $id ) ) {
			return;
		} elseif ( stag_is_photo( $id ) ) {
			$stag_post = new Stag_Photo( $id );
		} elseif ( stag_is_av( $id ) ) {
			$stag_post = new Stag_AV( $id );
		}
	}
endif;
add_action( 'the_post', 'stag_setup_data' );

/**
 * Register the required plugins for this theme.
 *
 * @since 1.0.1
 *
 * @return void
 */
function skilt_required_plugins() {
	$plugins = array(
		array(
			'name'     => __( 'StagTools', 'skilt' ),
			'slug'     => 'stagtools',
			'required' => true,
		),
		array(
			'name'             => __( 'Skilt Assistant', 'skilt' ),
			'slug'             => 'skilt-assistant',
			'source'           => get_parent_theme_file_path() . '/assistant/skilt-assistant.zip',
			'required'         => true,
			'force_activation' => true,
			'version'          => '1.0.0',
			'external_url'     => 'https://github.com/codestag/skilt-assistant/',
		),
	);

	tgmpa( $plugins );
}
add_action( 'tgmpa_register', 'skilt_required_plugins' );

if ( ! function_exists( 'skilt_assistant_notice' ) ) {
	/**
	 * Alerts if assistant plugin is not activated.
	 *
	 * @since 2.3.0
	 * @return void
	 */
	function skilt_assistant_notice() {
		if ( ! class_exists( 'Skilt_Assistant' ) && current_user_can( 'activate_plugins' ) ) {
			$page_link = self_admin_url( 'themes.php?page=tgmpa-install-plugins' );
			$message       = '<p>' . sprintf( '%1$s<br><strong>%2$s<a href="%3$s">this page</a>.', __( 'As per ThemeForest guidelines, some functionalities are now excluded from theme and moved to an assistant plugin.', 'skilt' ), __( 'To use Skilt with full features, please activate Skilt Assistant from ', 'skilt' ), esc_url( $page_link ) ) . '</strong></p>';

			echo '<div class="error">' . $message . '</div>'; // phpcs:ignore
		}
	}
}
add_action( 'admin_notices', 'skilt_assistant_notice' );

$temp_path = get_template_directory();

/**
 * Include Stag_Customizer class.
 */
require_once $temp_path . '/stag-customizer/stag-customizer.php';

/**
 * Include theme partials: widgets, metaboxes, and the rest.
 */
require_once $temp_path . '/inc/init.php';
require_once $temp_path . '/functions/content.php';
require_once $temp_path . '/functions/audio-video.php';
require_once $temp_path . '/functions/photo.php';

/**
* Include theme updater file
*/
require_once get_template_directory() . '/inc/updater/theme-updater.php';
