<?php
/**
 * Custom functions that act independently of the theme templates.
 *
 * Eventually, some of the functionality here could be replaced by core features.
 *
 * @package Stag_Customizer
 * @subpackage Ink
 */

/**
 * Get our wp_nav_menu() fallback, wp_page_menu(), to show a home link.
 *
 * @param array $args Configuration arguments.
 * @return array
 */
function stag_page_menu_args( $args ) {
	$args['show_home'] = true;
	return $args;
}
add_filter( 'wp_page_menu_args', 'stag_page_menu_args' );

/**
 * Filters wp_title to print a neat <title> tag based on what is being viewed.
 *
 * @param string $title Default title text for current view.
 * @param string $sep Optional separator.
 * @return string The filtered title.
 *
 * @deprecated 2.0.1
 */
function stag_wp_title( $title, $sep ) {
	if ( is_feed() ) {
			return $title;
	}

	global $page, $paged;

	// Add the blog name
	$title .= get_bloginfo( 'name', 'display' );

	// Add the blog description for the home/front page.
	$site_description = get_bloginfo( 'description', 'display' );
	if ( $site_description && ( is_home() || is_front_page() ) ) {
		$title .= " $sep $site_description";
	}

	// Add a page number if necessary:
	if ( ( $paged >= 2 || $page >= 2 ) && ! is_404() ) {
		$title .= " $sep " . sprintf( __( 'Page %s', 'skilt' ), max( $paged, $page ) );
	}

	return $title;
}
// add_filter( 'wp_title', 'stag_wp_title', 10, 2 );

if ( ! function_exists( 'stag_sanitize_text' ) ) :
	/**
	 * Allow only the allowedtags array in a string.
	 *
	 * @since  1.0.
	 *
	 * @param  string $string    The unsanitized string.
	 * @return string               The sanitized string.
	 */
	function stag_sanitize_text( $string ) {
		global $allowedtags;
		return wp_kses( $string, $allowedtags );
	}
endif;

if ( ! function_exists( 'stag_body_classes' ) ) :
	/**
	 * Adds custom classes to the array of body classes.
	 *
	 * @since  1.0.0.
	 *
	 * @param  array $classes    Classes for the body element.
	 * @return array                Modified class list.
	 */
	function stag_body_classes( $classes ) {
		// Determine if page has background

		$has_bg = false;

		if ( $background_image = get_theme_mod( 'background_image' ) ) {
			$has_bg = true;
		}

		if ( is_singular() && $background_image = stag_get_post_meta( 'settings', get_the_ID(), 'post-background-image' ) ) {
			$has_bg = true;
		}

		if ( $has_bg ) {
			$classes[] = 'has-background';
		}

		return $classes;
	}
endif;

add_filter( 'body_class', 'stag_body_classes' );

if ( ! function_exists( 'stag_post_classes' ) ) :
	/**
	 * Adds custom classes to the array of post classes.
	 *
	 * @param  array $classes Classes for the post.
	 * @return array          Modified class list.
	 */
	function stag_post_classes( $classes ) {
		if ( ! is_singular() ) {
			$classes[] = 'masonry-post';
		}

		if ( has_filter( 'stag_showing_related_posts' ) ) {
			$classes[] = 'masonry-post';
		}

		if ( is_singular() && '' === get_the_content( get_the_ID() ) ) {
			$classes[] = 'empty-content';
		}

		return $classes;
	}
endif;

add_filter( 'post_class', 'stag_post_classes' );

/**
 * Sets the authordata global when viewing an author archive.
 *
 * This provides backwards compatibility with
 * http://core.trac.wordpress.org/changeset/25574
 *
 * It removes the need to call the_post() and rewind_posts() in an author
 * template to print information about the author.
 *
 * @global WP_Query $wp_query WordPress Query object.
 * @return void
 */
function stag_setup_author() {
	global $wp_query;

	if ( $wp_query->is_author() && isset( $wp_query->post ) ) {
		$GLOBALS['authordata'] = get_userdata( $wp_query->post->post_author );
	}
}
add_action( 'wp', 'stag_setup_author' );

if ( ! function_exists( 'stag_sanitize_hex_color' ) ) :
	/**
	 * Validates a hex color.
	 *
	 * Returns either '', a 3 or 6 digit hex color (with #), or null.
	 * For validating values without a #, see sanitize_hex_color_no_hash().
	 *
	 * @since  1.0.
	 *
	 * @param  string $color    Hexidecimal value to sanitize.
	 * @return string|null              Sanitized value.
	 */
	function stag_sanitize_hex_color( $color ) {
		if ( '' === $color ) {
			return '';
		}

		// 3 or 6 hex digits, or the empty string.
		if ( preg_match( '|^#([A-Fa-f0-9]{3}){1,2}$|', $color ) ) {
			return $color;
		}

		return null;
	}
endif;

if ( ! function_exists( 'stag_sanitize_hex_color_no_hash' ) ) :
	/**
	 * Sanitizes a hex color without a hash. Use stag_sanitize_hex_color() when possible.
	 *
	 * Saving hex colors without a hash puts the burden of adding the hash on the
	 * UI, which makes it difficult to use or upgrade to other color types such as
	 * rgba, hsl, rgb, and html color names.
	 *
	 * Returns either '', a 3 or 6 digit hex color (without a #), or null.
	 *
	 * @since  1.0.
	 *
	 * @param  string $color    Hexidecimal value to sanitize.
	 * @return string|null              Sanitized value.
	 */
	function stag_sanitize_hex_color_no_hash( $color ) {
		$color = ltrim( $color, '#' );

		if ( '' === $color ) {
			return '';
		}

		return stag_sanitize_hex_color( '#' . $color ) ? $color : null;
	}
endif;

if ( ! function_exists( 'stag_maybe_hash_hex_color' ) ) :
	/**
	 * Ensures that any hex color is properly hashed.
	 * Otherwise, returns value untouched.
	 *
	 * This method should only be necessary if using sanitize_hex_color_no_hash().
	 *
	 * @since  1.0.
	 *
	 * @param  string $color    Hexidecimal value to sanitize.
	 * @return string              Sanitized value.
	 */
	function stag_maybe_hash_hex_color( $color ) {
		if ( $unhashed = stag_sanitize_hex_color_no_hash( $color ) ) {
			return '#' . $unhashed;
		}

		return $color;
	}
endif;

if ( ! function_exists( 'stag_jpg_quality' ) ) :
	/**
	 * Filter: Increase the quality of processed images.
	 *
	 * The default is 90. This sets it to the max of 100.
	 *
	 * @since 1.0.
	 *
	 * @return int
	 */
	function stag_jpg_quality() {
		return 100;
	}
endif;

add_filter( 'wp_editor_set_quality', 'stag_jpg_quality' );
add_filter( 'jpeg_quality', 'stag_jpg_quality' );

if ( ! function_exists( 'stag_excerpt_more' ) ) :
	/**
	 * Filter: Modify the suffix of a truncated excerpt.
	 *
	 * @since 1.0.
	 *
	 * @return string
	 */
	function stag_excerpt_more() {
		return _x( '&hellip;', 'end marker for truncated content', 'skilt' );
	}
endif;

add_filter( 'excerpt_more', 'stag_excerpt_more' );

if ( ! function_exists( 'stag_excerpt_length' ) ) :
	/**
	 * Set the excerpt length
	 *
	 * @since 1.0.
	 *
	 * @param  int $length The excerpt length
	 * @return int         The modified excerpt length
	 */
	function stag_excerpt_length( $length ) {
		global $post;

		$length = absint( stag_theme_mod( 'layout', 'excerpt-length' ) );

		return $length;
	}
endif;

add_filter( 'excerpt_length', 'stag_excerpt_length' );

/**
 * Return the WordPress array of allowed tags, with a few things added.
 *
 * @since 1.4.0.
 *
 * @return mixed|void
 */
function skilt_allowed_html() {
	$expandedtags = wp_kses_allowed_html();

	// Span.
	$expandedtags['span'] = array();

	// H1 - H6.
	$expandedtags['h1'] = array();
	$expandedtags['h2'] = array();
	$expandedtags['h3'] = array();
	$expandedtags['h4'] = array();
	$expandedtags['h5'] = array();
	$expandedtags['h6'] = array();

	// Enable id, class, and style attributes for each tag.
	foreach ( $expandedtags as $tag => $attributes ) {
		$expandedtags[ $tag ]['id']    = true;
		$expandedtags[ $tag ]['class'] = true;
		$expandedtags[ $tag ]['style'] = true;
	}

	// br (doesn't need attributes).
	$expandedtags['br'] = array();

	// img.
	$expandedtags['img'] = array(
		'src'    => true,
		'height' => true,
		'width'  => true,
		'alt'    => true,
		'title'  => true,
	);

	/**
	 * Customize the tags and attributes that are allows during text sanitization.
	 *
	 * @since 1.4.0.
	 *
	 * @param array     $expandedtags    The list of allowed tags and attributes.
	 * @param string    $string          The text string being sanitized.
	 */
	return apply_filters( 'skilt_allowed_html', $expandedtags );
}

/**
 * Return an array of allowed tags for Google Analytics Script.
 *
 * @since 1.4.0.
 *
 * @return mixed|void
 */
function skilt_analytics_allowed_code() {
	$expandedtags = array(
		'script' => array(
			'src'   => true,
			'async' => true,
		),
	);

	return $expandedtags;
}


/**
 * Sanitize for Google Analytics Script.
 *
 * @since 1.4.0.
 *
 * @return mixed|void
 */
function skilt_sanitize_analytics_code( $input ) {
	$value = '';

	if ( ! empty( $input ) ) {
		$value = wp_kses( $input, skilt_analytics_allowed_code() );
	}

	return $value;
}
