<?php
/**
 * @package Stag_Customizer
 */

if ( ! function_exists( 'stag_is_thumbnail_wide_enough' ) ) :
/**
 * Test an image in the Media Library to see if it meets a minimum width requirement.
 *
 * Returns the array of image info if it's wide enough, otherwise returns false.
 *
 * @since 1.0.
 *
 * @param  int    $id               The ID of the image.
 * @param  int    $minimum_width    The desired minimum width.
 * @param  string $size             The image size.
 * @return bool|array               Returns the array of image info if true, otherwise returns false
 */
function stag_is_thumbnail_wide_enough( $id = 0, $minimum_width = 1170, $size = 'full' ) {
	$clean_id = absint( $id );
	if ( 0 === $clean_id ) {
		return false;
	}

	// Sanitize minimum width parameter
	$minimum_width = absint( $minimum_width );
	if ( 0 === $minimum_width ) {
		global $content_width;
		$minimum_width = $content_width;
	}

	// Get the actual dimensions of the closest generated thumbnail size
	$image = wp_get_attachment_image_src( $clean_id, $size );

	// It's big
	if ( ! empty( $image ) && $minimum_width <= $image[1] ) {
		return $image;
	}

	// Not big
	return false;
}
endif;

if ( ! function_exists( 'stag_post_likes' ) ) :
/**
 * Displays post likes.
 *
 * @since 1.0.0.
 *
 * @return void
 */
function stag_post_likes( $post_id = null ) {
	if ( empty( $post_id ) ) {
		$post_id = get_the_ID();
	}

	if ( stag_theme_mod( 'layout', 'post-likes' ) ) return;

	$title      = __( 'Like this post', 'skilt' );
	$class      = 'post-likes';
	$post_likes = stag_get_post_meta( 'settings', $post_id, '_post-likes' );
	$user_id    = get_current_user_id();

	if( !$post_likes ) $post_likes = (int) 0;

	if ( stag_has_user_already_liked( $user_id, $post_id ) ) {
		$title = __( 'You already like this post', 'skilt' );
		$class .= ' active';
	}

	echo "<a href='#' title='{$title}' class='{$class}' id='post-like-{$post_id}'><span>". number_format_i18n( $post_likes ) ."</span> <i class='fas fa-heart post-like-icon'></i></a>";
}
endif;

if ( ! function_exists( 'stag_process_post_likes' ) ) :
/**
 * Process post likes.
 *
 * @since 1.0.0.
 *
 * @return void
 */
function stag_process_post_likes() {
	$data              = $_POST;
	$post_id           = absint( str_replace( 'post-like-', '', $data['likes_id'] ) );

	// Throw error and die, if post id not found
	if ( ! is_numeric( $post_id ) || $post_id === '' ) {
		wp_send_json_error();
	}

	$user_id           = get_current_user_id();
	$has_liked_already = stag_has_user_already_liked( $user_id, $post_id );
	$post_likes        = (int) stag_get_post_meta( 'settings', $post_id, '_post-likes' );

	// Check if post likes are blank and set it to 0.
	if ( ! $post_likes ) $post_likes = (int) 0;

	// Finally store the new value in post meta, and user meta, if user is logged in.
	if ( ! $has_liked_already ) {
		// Increment post likes by 1.
		$post_likes++;

		stag_update_post_meta( 'settings', $post_id, '_post-likes', $post_likes );

		if ( is_user_logged_in() ) {
			$liked = get_user_option( 'posts_liked', $user_id );
			if ( is_array( $liked ) ) {
				$liked[] = $post_id;
			} else {
				$liked = array( $post_id );
			}
			update_user_option( $user_id, 'posts_liked', $liked );
		} else {
			setcookie( $data['likes_id'], $post_id, time()*20, '/' );
		}
	}

	wp_send_json_success( array(
		'post_likes'        => number_format_i18n( $post_likes ),
		'has_liked_already' => $has_liked_already
	) );
}
endif;

add_action( 'wp_ajax_process_post_likes', 'stag_process_post_likes' );
add_action( 'wp_ajax_nopriv_process_post_likes', 'stag_process_post_likes' );

if ( ! function_exists( 'stag_has_user_already_liked' ) ) :
/**
 * Check if the current user has liked the given post.
 *
 * @param  int $user_id Current user ID.
 * @param  int $post_id Post ID, for which to determine the number of post likes.
 *
 * @since 1.0.0.
 *
 * @return bool True if user has already voted, else false.
 */
function stag_has_user_already_liked( $user_id, $post_id ) {
	$liked = get_user_option( 'posts_liked', $user_id );

	if ( is_array( $liked ) && in_array( $post_id, $liked ) ) {
		return true;
	}

	if( isset( $_COOKIE['post-like-' . $post_id ] ) ) {
		return true;
	}

	return false;
}
endif;

function stag_custom_user_fields() {

	$default_fields = apply_filters( 'stag_custom_user_fields', array(
		'twitter' => array(
			'title' => __( 'Twitter', 'skilt' ),
			'class' => 'fa-twitter',
		),
		'facebook' => array(
			'title' => __( 'Facebook', 'skilt' ),
			'class' => 'fa-facebook',
		),
		'instagram' => array(
			'title' => __( 'Instagram', 'skilt' ),
			'class' => 'fa-instagram',
		),
		'foursquare' => array(
			'title' => __( 'foursquare', 'skilt' ),
			'class' => 'fa-foursquare',
		),
		'google_plus_square' => array(
			'title' => __( 'Google+', 'skilt' ),
			'class' => 'fa-google-plus-square',
		)
	) );

	return $default_fields;
}

/**
 * Add custom user profile fields.
 *
 * @param  array $fields An array containing existing user profile fields.
 * @return array $fields
 */
function stag_user_profile_fields( $fields ) {

	$user_fields = stag_custom_user_fields();

	foreach( $user_fields as $key => $social ) {
		$fields[$key] = $social['title'];
	}

	return $fields;
}
add_filter( 'user_contactmethods', 'stag_user_profile_fields' );

/**
 * Retrieve user's custom social profile.
 *
 * @param  int $id User ID.
 * @return array Custom user fields.
 */
function stag_get_user_social_profiles( $id ) {
	$fields      = array();
	$user_fields = stag_custom_user_fields();

	foreach( $user_fields as $field_key => $field_value ) {
		$fields[$field_key] = get_the_author_meta( $field_key, $id );
	}

	return array_filter( $fields );
}
